/* This audio streaming and processing architecture is based on the Tympan example sketch "MyAudioAlgorithm"
 */

#include "AudioStream_F32.h"
#include "spu.h"

//Class definition
class ProcessAudio_F32 : public AudioStream_F32 {
public:

  //constructor
  ProcessAudio_F32(const AudioSettings_F32 &settings)
    : AudioStream_F32(1, inputQueueArray_f32) {
    ;
  };

  //This is called every time a new chunk of audio is received
  void update(void) {

    //get audio block
    audio_block_f32_t *audio_block;
    audio_block = AudioStream_F32::receiveWritable_f32();
    if (!audio_block) return;

    //process audio
    process(audio_block);

    ///transmit the block and release memory
    AudioStream_F32::transmit(audio_block);
    AudioStream_F32::release(audio_block);
  }

  //This is called to process the actual block of audio samples
  void process(audio_block_f32_t *audio_block) {

    //only process audio with AINR algorithm if not in loopback mode
    if (!loopback_mode) {

      //create buffer for communicating with SPU
      uint32_t SPUBuffer[HOP_SIZE / 4 * 2 + 2];
      uint32_t SPUBuffer2[HOP_SIZE / 4 * 2 + 2];

      //set route and PC data
      SPUBuffer[0] = 0x00000000;  //route
      SPUBuffer[1] = 0x00000000;  //PC (anything works in this case)

      //pack samples into buffer to send to SPU
      int j = 0;
      for (int i = 0; i < (HOP_SIZE / 4 * 4); i += 4) {
        int32_t samples[4];
        for (int k = 0; k < 4; k++)
          samples[k] = (audio_block->data[i + k]) * 32767; //convert to integer

        SPUBuffer[2 + j++] = (samples[0] & 0x0000FFFF) | ((samples[1] << 16) & 0xFFFF0000);
        SPUBuffer[2 + j++] = (samples[2] & 0x0000FFFF) | ((samples[3] << 16) & 0xFFFF0000);
      }

      //send samples to SPU
      SPU_Write(0x03, 0, (HOP_SIZE / 4 * 2 + 2), SPUBuffer);

      //wait some time for SPU to process samples. A good starting point is 4500us for 16ms models, and 2500us for 8ms models (100MHz clock)
      //Note: you may not wait longer than a value close to the hop size time
      delayMicroseconds(4500);
      
      //read response from SPU
      SPU_Read(0x03, 0, (HOP_SIZE / 4 * 2 + 2), SPUBuffer2); //the response from this model is the AINR filtered audio
      
      //unpack samples from buffer received from SPU
      j = 0;
      for (int i = 2; i < (HOP_SIZE / 4 * 2 + 2); i += 2) {
        int16_t samples[4];
        samples[0] = (SPUBuffer2[i] & 0x0000FFFF) >> 0;     //unpack 16 bit sample
        audio_block->data[j++] = 0.0000305185 * samples[0]; //convert to floating point

        samples[1] = ((SPUBuffer2[i] & 0xFFFF0000) >> 16);
        audio_block->data[j++] = 0.0000305185 * samples[1];

        samples[2] = (SPUBuffer2[i + 1] & 0x0000FFFF) >> 0;
        audio_block->data[j++] = 0.0000305185 * samples[2]; 

        samples[3] = ((SPUBuffer2[i + 1] & 0xFFFF0000) >> 16);
        audio_block->data[j++] = 0.0000305185 * samples[3];
      }
    }
  }


  //sets loopback_mode tracker
  float32_t setLoopbackMode(int setting) {
    return loopback_mode = setting;
  }

private:
  audio_block_f32_t *inputQueueArray_f32[1];  //memory pointer for the input to this module

  int loopback_mode = 1;  //keep track of loopback_mode
};
