#ifndef SPU_H
#define SPU_H

#include "Arduino.h"
#include <SPI.h>

#define SPI_SPEED 4000000 

#define SPU_RESET_PIN 32

//Hop size (number of samples) of SPU input
#define HOP_SIZE 128

/*Initializes the SPU
 *returns 0 if registers cannot be read
 *returns -1 if PLL does not lock
 *returns 1 if everything is okay
 */
int initializeSPU();

//Resets the SPU using the reset pin
void SPUReset(int resetPin);

//Sets up the SPU PLL registers
int SPUPLLSetup();

//Turns on all of the SPU memory banks
void SPUMemorySetup();

/* Request to read SPU data.
 * instruction: SPU instruction 
 * address: start address of data requested
 * length: length of data requested
 * data: buffer for returned data from SPU
 */
void SPU_Read(byte instruction, uint32_t address, int length, uint32_t* data);

/* Write data to SPU.
 * instruction: SPU instruction 
 * address: start address of write location
 * length: length of data to write
 * data: buffer of data to write to SPU
 */
void SPU_Write(byte instruction, uint32_t address, int length, uint32_t* data);

//Tests basic SPU register reads
int SPUIntegrityCheck();

#endif