/* This audio streaming and processing architecture is based on the Tympan example sketch "MyAudioAlgorithm"
 */

#include "AudioStream_F32.h"
#include "spu.h"

//IO pins for onboard LEDs
#define LED_RED 16
#define LED_GREEN 15

//Class definition
class ProcessAudio_F32 : public AudioStream_F32 {
public:

  //constructor
  ProcessAudio_F32(const AudioSettings_F32 &settings)
    : AudioStream_F32(1, inputQueueArray_f32) {
    ;
  };

  //This is called every time a new chunk of audio is received
  void update(void) {

    //get audio block
    audio_block_f32_t *audio_block;
    audio_block = AudioStream_F32::receiveWritable_f32();
    if (!audio_block) return;

    //process audio
    process(audio_block);

    //transmit the block and release memory
    AudioStream_F32::transmit(audio_block);
    AudioStream_F32::release(audio_block);
  }

  //This is called to process the actual block of audio samples
  void process(audio_block_f32_t *audio_block) {

    //create buffer for communicating with SPU
    uint32_t SPUBuffer[HOP_SIZE / 4 * 2 + 2];

    //set route and PC data
    SPUBuffer[0] = 0x00000000;  //route
    SPUBuffer[1] = 0x00000000;  //PC (anything works in this case)

    //pack samples into buffer to send to SPU
    int j = 0;
    for (int i = 0; i < (HOP_SIZE / 4 * 4); i += 4) {
      int32_t samples[4];
      for (int k = 0; k < 4; k++)
        samples[k] = (audio_block->data[i + k]) * 32767; //convert to integer

      SPUBuffer[2 + j++] = (samples[0] & 0x0000FFFF) | ((samples[1] << 16) & 0xFFFF0000);
      SPUBuffer[2 + j++] = (samples[2] & 0x0000FFFF) | ((samples[3] << 16) & 0xFFFF0000);
    }

    //send samples to SPU
    SPU_Write(0x03, 0, (HOP_SIZE / 4 * 2 + 2), SPUBuffer);

    //wait some time for SPU to process samples
    delayMicroseconds(10000);
    
    //create buffer for communicating with SPU
    uint32_t SPUBuffer2[6]; //the response from this model is always 6 words long

    //read response from SPU
    SPU_Read(0x03, 0, 6, SPUBuffer2);

    //for debugging: print value of SPU response
    /*if ((SPUBuffer2[2] >> 15) == 0) {
      Serial.println(SPUBuffer2[2]);
    } else {
      int val = SPUBuffer2[2];
      val = ~val + 1;
      val &= 0x0000FFFF;
      Serial.println(-val);
    }*/

    //if response is a positive number, the phrase was detected.
    if ((SPUBuffer2[2] >> 15) == 0) {

      //print to serial terminal
      Serial.print(millis());
      Serial.print(": ");
      Serial.println("Hey Snips!");

      //flash LED
      digitalWrite(LED_RED, HIGH);
      delay(30);
      digitalWrite(LED_RED, LOW);
      delay(30);
      digitalWrite(LED_GREEN, HIGH);
      delay(30);
      digitalWrite(LED_GREEN, LOW);
      delay(30);
    }
  }

private:
  audio_block_f32_t *inputQueueArray_f32[1];  //memory pointer for the input to this module
};
